
package ch.capi.utils;

import java.util.*;
import java.util.zip.*;
import java.util.jar.*;
import java.io.*;
import java.net.*;

public class PackageUtils
{
	public static List<Class<?>> getClassesFromPackage(String packageName)
	{
		List<Class<?>> classes = new LinkedList<Class<?>>();
		
		//ciblage du package voulu
		String targetPackagePath = packageName.replace('.', '/');
		String targetPath = "/"+targetPackagePath;
		 
		// récupération du chemin 'réel' du package
		URL resourceURL = PackageUtils.class.getResource(targetPath);
		if (resourceURL == null)
		{
			throw new RuntimeException("Unable to find the resource "+targetPath);
		}
		
		String packageRealPath = resourceURL.getFile();
		File packageFile = new File(packageRealPath);

		// si le fichier exist, alors aucun problème		
		if (packageFile.exists())
		{
			for (File classFile : packageFile.listFiles())
			{
			   String fileName = classFile.getName();
			   if (fileName.endsWith(".class")) // ne traite que les fichiers '.class'
			   {
				  Class<?> clazz = getClass(packageName, fileName);
				  classes.add(clazz);
			   }
			}
		}
		
		// c'est un fichier jar
		else if (packageRealPath.contains("jar!"))
		{
			String jarPath = packageRealPath.substring(0, packageRealPath.indexOf("jar!")+3);
			
			try
			{
				JarFile jf = new JarFile(jarPath);
				Enumeration<JarEntry> entries = jf.entries();
				
				// loop sur toutes les entrées du Jar pour voir si le nom
				// commence avec le package voulu
				while (entries.hasMoreElements())
				{
				   JarEntry entry = entries.nextElement();
				   String entryName = entry.getName();

				   if (entryName.startsWith(targetPackagePath) && 
					   entryName.endsWith(".class"))
				   {
					  Class<?> clazz = getClass(packageName, entryName);
					  classes.add(clazz);
				   }
				}
			}
			catch(Exception e)
			{
				throw new RuntimeException("Unable to read from file : "+jarPath, e);
			}
		}
		
		// on sait pas ce que c'est...
		else
		{
			throw new RuntimeException("Unknown file type : "+packageRealPath);
		}
		
		return classes;
	}
	
	private static Class<?> getClass(String packageName, String fileName)
	{
		String className = fileName.substring(0, fileName.lastIndexOf(".")); // enlève le '.class'
		String fullClassName = packageName+"."+className;
		
		try
		{
			return Class.forName(fullClassName);
		}
		catch (ClassNotFoundException cnfe)
		{
			throw new RuntimeException("Unable to find class "+fullClassName, cnfe);
		}
	}
	
	public static void main(String[] args)
	{
		List<Class<?>> classes = getClassesFromPackage(args[0]);
		for (Class<?> clazz : classes)
		{
			System.out.println("found : "+clazz.getName());
		}
	}
}
